@@ -28,19 +28,30 @@
 #define AST_CACHE_SLOT_LINENO   &AST_G(cache_slots)[2 * 2]
 #define AST_CACHE_SLOT_CHILDREN &AST_G(cache_slots)[2 * 3]
 
-#define AST_CURRENT_VERSION 20
+#define AST_CURRENT_VERSION 35
 
 /* Additional flags for BINARY_OP */
 #define AST_BINARY_IS_GREATER 256
 #define AST_BINARY_IS_GREATER_OR_EQUAL 257
 #define AST_BINARY_BOOL_OR 258
 #define AST_BINARY_BOOL_AND 259
+#define AST_BINARY_COALESCE 260
 
 /* Flags for UNARY_OP to use instead of AST_SILENCE, AST_UNARY_PLUS, AST_UNARY_MINUS */
 #define AST_SILENCE 260
 #define AST_PLUS 261
 #define AST_MINUS 262
 
+/* Define some constants for PHP 7.0 */
+#if PHP_VERSION_ID < 70100
+# define IS_VOID 18
+# define IS_ITERABLE 19
+# define ZEND_TYPE_NULLABLE (1<<8)
+# define ZEND_ARRAY_SYNTAX_LIST 1
+# define ZEND_ARRAY_SYNTAX_LONG 2
+# define ZEND_ARRAY_SYNTAX_SHORT 3
+#endif
+
 static inline void ast_update_property(zval *object, zend_string *name, zval *value, void **cache_slot) {
 	zval name_zv;
 	ZVAL_STR(&name_zv, name);
@@ -96,7 +107,8 @@ static inline zend_bool ast_kind_uses_attr(zend_ast_kind kind) {
 		|| kind == ZEND_AST_CAST || kind == ZEND_AST_MAGIC_CONST || kind == ZEND_AST_ARRAY_ELEM
 		|| kind == ZEND_AST_INCLUDE_OR_EVAL || kind == ZEND_AST_USE || kind == ZEND_AST_PROP_DECL
 		|| kind == ZEND_AST_GROUP_USE || kind == ZEND_AST_USE_ELEM
-		|| kind == AST_NAME || kind == AST_CLOSURE_VAR;
+		|| kind == AST_NAME || kind == AST_CLOSURE_VAR || kind == ZEND_AST_CLASS_CONST_DECL
+		|| kind == ZEND_AST_ARRAY;
 }
 
 static inline zend_bool ast_kind_is_decl(zend_ast_kind kind) {
@@ -134,11 +146,75 @@ static inline zend_bool ast_is_name(zend_ast *ast, zend_ast *parent, uint32_t i)
 	return 0;
 }
 
+/* Assumes that ast_is_name is already true */
+static inline zend_bool ast_is_type(zend_ast *ast, zend_ast *parent, uint32_t i) {
+	if (i == 0) {
+		return parent->kind == ZEND_AST_PARAM;
+	}
+	if (i == 3) {
+		return parent->kind == ZEND_AST_CLOSURE || parent->kind == ZEND_AST_FUNC_DECL
+			|| parent->kind == ZEND_AST_METHOD;
+	}
+	return 0;
+}
+
 static inline zend_bool ast_is_var_name(zend_ast *ast, zend_ast *parent, uint32_t i) {
 	return (parent->kind == ZEND_AST_STATIC && i == 0)
 		|| (parent->kind == ZEND_AST_CATCH && i == 1);
 }
 
+/* Whether this node may need statement list normalization */
+static inline zend_bool ast_should_normalize_list(zend_ast *ast, zend_ast *parent, uint32_t i) {
+	if (ast && ast->kind == ZEND_AST_STMT_LIST) {
+		return 0;
+	}
+
+	if (i == 0) {
+		return parent->kind == ZEND_AST_DO_WHILE;
+	}
+	if (i == 1) {
+		if (parent->kind == ZEND_AST_DECLARE) {
+			/* declare(); and declare() {} are not the same */
+			return ast != NULL;
+		}
+		return parent->kind == ZEND_AST_IF_ELEM || parent->kind == ZEND_AST_WHILE;
+	}
+	if (i == 2) {
+		return parent->kind == ZEND_AST_CATCH;
+	}
+	if (i == 3) {
+		return parent->kind == ZEND_AST_FOR || parent->kind == ZEND_AST_FOREACH;
+	}
+	return 0;
+}
+
+/* Adopted from zend_compile.c */
+typedef struct _builtin_type_info {
+	const char* name;
+	const size_t name_len;
+	const zend_uchar type;
+} builtin_type_info;
+static const builtin_type_info builtin_types[] = {
+	{ZEND_STRL("int"), IS_LONG},
+	{ZEND_STRL("float"), IS_DOUBLE},
+	{ZEND_STRL("string"), IS_STRING},
+	{ZEND_STRL("bool"), _IS_BOOL},
+	{ZEND_STRL("void"), IS_VOID},
+	{ZEND_STRL("iterable"), IS_ITERABLE},
+	{NULL, 0, IS_UNDEF}
+};
+static inline zend_uchar lookup_builtin_type(const zend_string *name) {
+	const builtin_type_info *info = &builtin_types[0];
+	for (; info->name; ++info) {
+		if (ZSTR_LEN(name) == info->name_len
+			&& !zend_binary_strcasecmp(ZSTR_VAL(name), ZSTR_LEN(name), info->name, info->name_len)
+		) {
+			return info->type;
+		}
+	}
+	return 0;
+}
+
 static inline zend_ast_attr ast_assign_op_to_binary_op(zend_ast_attr attr) {
 	switch (attr) {
 		case ZEND_ASSIGN_BW_OR: return ZEND_BW_OR;
@@ -157,6 +233,22 @@ static inline zend_ast_attr ast_assign_op_to_binary_op(zend_ast_attr attr) {
 	}
 }
 
+static inline zend_bool ast_array_is_list(zend_ast *ast) {
+	zend_ast_list *list = zend_ast_get_list(ast);
+	uint32_t i;
+	if (ast->attr != ZEND_ARRAY_SYNTAX_LIST) {
+		return 0;
+	}
+
+	for (i = 0; i < list->children; i++) {
+		if (list->child[i]->child[1] != NULL || list->child[i]->attr) {
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
 static inline zend_ast **ast_get_children(zend_ast *ast, uint32_t *count) {
 	if (ast_kind_is_decl(ast->kind)) {
 		zend_ast_decl *decl = (zend_ast_decl *) ast;
@@ -172,33 +264,49 @@ static inline zend_ast **ast_get_children(zend_ast *ast, uint32_t *count) {
 	}
 }
 
-static void ast_create_virtual_node(
-		zval *zv, zend_ast_kind kind, zend_ast *ast, zend_long version) {
-	zval tmp_zv, tmp_zv2;
+static void ast_to_zval(zval *zv, zend_ast *ast, zend_long version);
+
+static void ast_create_virtual_node_ex(
+		zval *zv, zend_ast_kind kind, zend_ast_attr attr, uint32_t lineno,
+		zend_long version, uint32_t num_children, ...) {
+	zval tmp_zv;
+	va_list va;
+	uint32_t i;
 
 	object_init_ex(zv, ast_node_ce);
 
 	ZVAL_LONG(&tmp_zv, kind);
 	ast_update_property(zv, AST_STR(str_kind), &tmp_zv, AST_CACHE_SLOT_KIND);
 
-	ZVAL_LONG(&tmp_zv, ast->attr);
+	ZVAL_LONG(&tmp_zv, attr);
 	ast_update_property(zv, AST_STR(str_flags), &tmp_zv, AST_CACHE_SLOT_FLAGS);
 
-	ZVAL_LONG(&tmp_zv, zend_ast_get_lineno(ast));
+	ZVAL_LONG(&tmp_zv, lineno);
 	ast_update_property(zv, AST_STR(str_lineno), &tmp_zv, AST_CACHE_SLOT_LINENO);
 
 	array_init(&tmp_zv);
 	ast_update_property(zv, AST_STR(str_children), &tmp_zv, AST_CACHE_SLOT_CHILDREN);
 
-	ZVAL_COPY(&tmp_zv2, zend_ast_get_zval(ast));
-	if (version >= 30) {
-		zend_hash_add_new(Z_ARRVAL(tmp_zv), ast_kind_child_name(kind, 0), &tmp_zv2);
-	} else {
-		zend_hash_next_index_insert(Z_ARRVAL(tmp_zv), &tmp_zv2);
+	va_start(va, num_children);
+	for (i = 0; i < num_children; i++) {
+		zval *child_zv = va_arg(va, zval *);
+		zend_string *child_name = version >= 30 ? ast_kind_child_name(kind, i) : NULL;
+		if (child_name) {
+			zend_hash_add_new(Z_ARRVAL(tmp_zv), child_name, child_zv);
+		} else {
+			zend_hash_next_index_insert(Z_ARRVAL(tmp_zv), child_zv);
+		}
 	}
+	va_end(va);
 }
 
-static void ast_to_zval(zval *zv, zend_ast *ast, zend_long version);
+static void ast_create_virtual_node(
+		zval *zv, zend_ast_kind kind, zend_ast *child, zend_long version) {
+	zval child_zv;
+	ast_to_zval(&child_zv, child, version);
+	return ast_create_virtual_node_ex(
+		zv, kind, child->attr, zend_ast_get_lineno(child), version, 1, &child_zv);
+}
 
 static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, zend_long version) {
 	uint32_t i, count;
@@ -210,21 +318,102 @@ static void ast_fill_children_ht(HashTable *ht, zend_ast *ast, zend_long version
 			!is_list && version >= 30 ? ast_kind_child_name(ast->kind, i) : NULL;
 		zval child_zv;
 
-		if (version >= 20 && ast->kind == ZEND_AST_STMT_LIST
-				&& child != NULL && child->kind == ZEND_AST_STMT_LIST) {
-			ast_fill_children_ht(ht, child, version);
-			continue;
+		if (version >= 20 && ast->kind == ZEND_AST_STMT_LIST) {
+			if (child != NULL && child->kind == ZEND_AST_STMT_LIST) {
+				ast_fill_children_ht(ht, child, version);
+				continue;
+			}
+			if (version >= 40 && child == NULL) {
+				continue;
+			}
 		}
 
-		if (ast_is_name(child, ast, i)) {
-			ast_create_virtual_node(&child_zv, AST_NAME, child, version);
+		/* These two AST_CATCH checks should occur before ast_is_name() */
+#if PHP_VERSION_ID >= 70100
+		if (ast->kind == ZEND_AST_CATCH && version < 35
+				&& i == 0 && zend_ast_get_list(child)->children == 1) {
+			/* Emulate PHP 7.0 format (no list) */
+			ast_create_virtual_node(
+				&child_zv, AST_NAME, zend_ast_get_list(child)->child[0], version);
+		}
+#else
+		if (ast->kind == ZEND_AST_CATCH && version >= 35 && i == 0) {
+			/* Emulate PHP 7.1 format (name list) */
+			zval tmp;
+			ast_create_virtual_node(&tmp, AST_NAME, child, version);
+			ast_create_virtual_node_ex(
+				&child_zv, ZEND_AST_NAME_LIST, 0, zend_ast_get_lineno(child), version, 1, &tmp);
+		}
+#endif
+		else if (ast_is_name(child, ast, i)) {
+			zend_uchar type;
+			zend_bool is_nullable = 0;
+			if (child->attr & ZEND_TYPE_NULLABLE) {
+				is_nullable = 1;
+				child->attr &= ~ZEND_TYPE_NULLABLE;
+			}
+
+			if (version >= 40 && child->attr == ZEND_NAME_FQ) {
+				/* Ensure there is no leading \ for fully-qualified names. This can happen if
+				 * something like ('\bar')() is used. */
+				zval *name = zend_ast_get_zval(child);
+				if (Z_STRVAL_P(name)[0] == '\\') {
+					zend_string *new_name = zend_string_init(
+						Z_STRVAL_P(name) + 1, Z_STRLEN_P(name) - 1, 0);
+					zend_string_release(Z_STR_P(name));
+					Z_STR_P(name) = new_name;
+				}
+			}
+
+			if (version >= 40 && child->attr == ZEND_NAME_NOT_FQ && ast_is_type(child, ast, i)
+					&& (type = lookup_builtin_type(zend_ast_get_str(child)))) {
+				/* Convert "int" etc typehints to TYPE nodes */
+				ast_create_virtual_node_ex(
+					&child_zv, ZEND_AST_TYPE, type, zend_ast_get_lineno(child), version, 0);
+			} else {
+				ast_create_virtual_node(&child_zv, AST_NAME, child, version);
+			}
+
+			if (is_nullable) {
+				/* Create explicit AST_NULLABLE_TYPE node */
+				zval tmp;
+				ZVAL_COPY_VALUE(&tmp, &child_zv);
+				ast_create_virtual_node_ex(
+					&child_zv, AST_NULLABLE_TYPE, 0, zend_ast_get_lineno(child), version, 1, &tmp);
+			}
 		} else if (ast->kind == ZEND_AST_CLOSURE_USES) {
 			ast_create_virtual_node(&child_zv, AST_CLOSURE_VAR, child, version);
 		} else if (version >= 20 && ast_is_var_name(child, ast, i)) {
 			ast_create_virtual_node(&child_zv, ZEND_AST_VAR, child, version);
-		} else if (ast->kind == ZEND_AST_PROP_ELEM && i == 2) {
+		} else if (version >= 40 && ast_should_normalize_list(child, ast, i)) {
+			if (child) {
+				zval tmp;
+				ast_to_zval(&tmp, child, version);
+				ast_create_virtual_node_ex(
+					&child_zv, ZEND_AST_STMT_LIST, 0, zend_ast_get_lineno(child), version, 1, &tmp);
+			} else {
+				ast_create_virtual_node_ex(
+					&child_zv, ZEND_AST_STMT_LIST, 0, zend_ast_get_lineno(ast), version, 0);
+			}
+		} else if (i == 2
+				&& (ast->kind == ZEND_AST_PROP_ELEM || ast->kind == ZEND_AST_CONST_ELEM)) {
 			/* Skip docComment child -- It's handled separately */
 			continue;
+#if PHP_VERSION_ID >= 70100
+		} else if (ast->kind == ZEND_AST_LIST) {
+			/* Emulate simple variable list */
+			ast_to_zval(&child_zv, child->child[0], version);
+#else
+		} else if (version >= 35 && ast->kind == ZEND_AST_ARRAY
+				&& ast->attr == ZEND_ARRAY_SYNTAX_LIST) {
+			/* Emulate ARRAY_ELEM list */
+			zval ch0, ch1;
+			ast_to_zval(&ch0, child, version);
+			ZVAL_NULL(&ch1);
+			ast_create_virtual_node_ex(
+				&child_zv, ZEND_AST_ARRAY_ELEM, 0, zend_ast_get_lineno(child), version,
+				2, &ch0, &ch1);
+#endif
 		} else {
 			ast_to_zval(&child_zv, child, version);
 		}
@@ -273,6 +462,12 @@ static void ast_to_zval(zval *zv, zend_ast *ast, zend_long version) {
 				ast->kind = ZEND_AST_BINARY_OP;
 				ast->attr = AST_BINARY_BOOL_AND;
 				break;
+			case ZEND_AST_COALESCE:
+				if (version >= 40) {
+					ast->kind = ZEND_AST_BINARY_OP;
+					ast->attr = AST_BINARY_COALESCE;
+				}
+				break;
 			case ZEND_AST_SILENCE:
 				ast->kind = ZEND_AST_UNARY_OP;
 				ast->attr = AST_SILENCE;
@@ -288,6 +483,18 @@ static void ast_to_zval(zval *zv, zend_ast *ast, zend_long version) {
 		}
 	}
 
+#if PHP_VERSION_ID >= 70100
+	if (version < 35 && ast->kind == ZEND_AST_ARRAY && ast_array_is_list(ast)) {
+		ast->kind = ZEND_AST_LIST;
+		ast->attr = 0;
+	}
+#else
+	if (version >= 35 && ast->kind == ZEND_AST_LIST) {
+		ast->kind = ZEND_AST_ARRAY;
+		ast->attr = ZEND_ARRAY_SYNTAX_LIST;
+	}
+#endif
+
 	is_decl = ast_kind_is_decl(ast->kind);
 	object_init_ex(zv, is_decl ? ast_decl_ce : ast_node_ce);
 
@@ -320,6 +527,11 @@ static void ast_to_zval(zval *zv, zend_ast *ast, zend_long version) {
 		}
 		ast_update_property(zv, AST_STR(str_docComment), &tmp_zv, NULL);
 	} else {
+#if PHP_VERSION_ID < 70100
+		if (ast->kind == ZEND_AST_CLASS_CONST_DECL) {
+			ast->attr = ZEND_ACC_PUBLIC;
+		}
+#endif
 		ZVAL_LONG(&tmp_zv, ast->attr);
 		ast_update_property(zv, AST_STR(str_flags), &tmp_zv, AST_CACHE_SLOT_FLAGS);
 	}
@@ -338,10 +550,17 @@ static void ast_to_zval(zval *zv, zend_ast *ast, zend_long version) {
 		}
 	}
 
+	/* Convert doc comments on properties and constants into properties */
 	if (version >= 15 && ast->kind == ZEND_AST_PROP_ELEM && ast->child[2]) {
 		ZVAL_STR_COPY(&tmp_zv, zend_ast_get_str(ast->child[2]));
 		ast_update_property(zv, AST_STR(str_docComment), &tmp_zv, NULL);
 	}
+#if PHP_VERSION_ID >= 70100
+	if (ast->kind == ZEND_AST_CONST_ELEM && ast->child[2]) {
+		ZVAL_STR_COPY(&tmp_zv, zend_ast_get_str(ast->child[2]));
+		ast_update_property(zv, AST_STR(str_docComment), &tmp_zv, NULL);
+	}
+#endif
 
 	array_init(&tmp_zv);
 	ast_update_property(zv, AST_STR(str_children), &tmp_zv, AST_CACHE_SLOT_CHILDREN);
@@ -349,7 +568,7 @@ static void ast_to_zval(zval *zv, zend_ast *ast, zend_long version) {
 	ast_fill_children_ht(Z_ARRVAL(tmp_zv), ast, version);
 }
 
-static const zend_long versions[] = {10, 15, 20, 30};
+static const zend_long versions[] = {10, 15, 20, 30, 35, 40};
 static const size_t versions_count = sizeof(versions)/sizeof(versions[0]);
 
 static zend_string *ast_version_info() {
@@ -369,15 +588,26 @@ static zend_string *ast_version_info() {
 	return str.s;
 }
 
-static int ast_check_version(zend_long version) {
+static inline zend_bool ast_version_known(zend_long version) {
 	size_t i;
-	zend_string *version_info;
-
 	for (i = 0; i < versions_count; ++i) {
 		if (version == versions[i]) {
-			return SUCCESS;
+			return 1;
 		}
 	}
+	return 0;
+}
+
+static int ast_check_version(zend_long version) {
+	zend_string *version_info;
+
+	if (ast_version_known(version)) {
+		if (version == 10 || version == 15 || version == 20) {
+			php_error_docref(NULL, E_DEPRECATED,
+				"Version " ZEND_LONG_FMT " is deprecated", version);
+		}
+		return SUCCESS;
+	}
 
 	version_info = ast_version_info();
 	if (version != -1) {
@@ -566,6 +796,8 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("TYPE_ARRAY", IS_ARRAY);
 	ast_register_flag_constant("TYPE_OBJECT", IS_OBJECT);
 	ast_register_flag_constant("TYPE_CALLABLE", IS_CALLABLE);
+	ast_register_flag_constant("TYPE_VOID", IS_VOID);
+	ast_register_flag_constant("TYPE_ITERABLE", IS_ITERABLE);
 
 	ast_register_flag_constant("UNARY_BOOL_NOT", ZEND_BOOL_NOT);
 	ast_register_flag_constant("UNARY_BITWISE_NOT", ZEND_BW_NOT);
@@ -597,6 +829,7 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("BINARY_IS_GREATER", AST_BINARY_IS_GREATER);
 	ast_register_flag_constant("BINARY_IS_GREATER_OR_EQUAL", AST_BINARY_IS_GREATER_OR_EQUAL);
 	ast_register_flag_constant("BINARY_SPACESHIP", ZEND_SPACESHIP);
+	ast_register_flag_constant("BINARY_COALESCE", AST_BINARY_COALESCE);
 
 	ast_register_flag_constant("ASSIGN_BITWISE_OR", ZEND_ASSIGN_BW_OR);
 	ast_register_flag_constant("ASSIGN_BITWISE_AND", ZEND_ASSIGN_BW_AND);
@@ -630,6 +863,10 @@ PHP_MINIT_FUNCTION(ast) {
 	ast_register_flag_constant("MAGIC_CLASS", T_CLASS_C);
 	ast_register_flag_constant("MAGIC_TRAIT", T_TRAIT_C);
 
+	ast_register_flag_constant("ARRAY_SYNTAX_LIST", ZEND_ARRAY_SYNTAX_LIST);
+	ast_register_flag_constant("ARRAY_SYNTAX_LONG", ZEND_ARRAY_SYNTAX_LONG);
+	ast_register_flag_constant("ARRAY_SYNTAX_SHORT", ZEND_ARRAY_SYNTAX_SHORT);
+
 	INIT_CLASS_ENTRY(tmp_ce, "ast\\Node", NULL);
 	ast_node_ce = zend_register_internal_class(&tmp_ce);
 
