@@ -1,12 +1,8 @@
 #include "php_ast.h"
 
-const size_t ast_kinds_count = 98;
+const size_t ast_kinds_count = 99;
 
 const zend_ast_kind ast_kinds[] = {
-	ZEND_AST_FUNC_DECL,
-	ZEND_AST_CLOSURE,
-	ZEND_AST_METHOD,
-	ZEND_AST_CLASS,
 	ZEND_AST_ARG_LIST,
 	ZEND_AST_LIST,
 	ZEND_AST_ARRAY,
@@ -24,6 +20,13 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_NAME_LIST,
 	ZEND_AST_TRAIT_ADAPTATIONS,
 	ZEND_AST_USE,
+	AST_NAME,
+	AST_CLOSURE_VAR,
+	AST_NULLABLE_TYPE,
+	ZEND_AST_FUNC_DECL,
+	ZEND_AST_CLOSURE,
+	ZEND_AST_METHOD,
+	ZEND_AST_CLASS,
 	ZEND_AST_MAGIC_CONST,
 	ZEND_AST_TYPE,
 	ZEND_AST_VAR,
@@ -82,6 +85,7 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_SWITCH,
 	ZEND_AST_SWITCH_CASE,
 	ZEND_AST_DECLARE,
+	ZEND_AST_PROP_ELEM,
 	ZEND_AST_CONST_ELEM,
 	ZEND_AST_USE_TRAIT,
 	ZEND_AST_TRAIT_PRECEDENCE,
@@ -96,19 +100,12 @@ const zend_ast_kind ast_kinds[] = {
 	ZEND_AST_TRY,
 	ZEND_AST_CATCH,
 	ZEND_AST_PARAM,
-	ZEND_AST_PROP_ELEM,
 	ZEND_AST_FOR,
 	ZEND_AST_FOREACH,
-	AST_NAME,
-	AST_CLOSURE_VAR,
 };
 
 const char *ast_kind_to_name(zend_ast_kind kind) {
 	switch (kind) {
-		case ZEND_AST_FUNC_DECL: return "AST_FUNC_DECL";
-		case ZEND_AST_CLOSURE: return "AST_CLOSURE";
-		case ZEND_AST_METHOD: return "AST_METHOD";
-		case ZEND_AST_CLASS: return "AST_CLASS";
 		case ZEND_AST_ARG_LIST: return "AST_ARG_LIST";
 		case ZEND_AST_LIST: return "AST_LIST";
 		case ZEND_AST_ARRAY: return "AST_ARRAY";
@@ -126,6 +123,13 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_NAME_LIST: return "AST_NAME_LIST";
 		case ZEND_AST_TRAIT_ADAPTATIONS: return "AST_TRAIT_ADAPTATIONS";
 		case ZEND_AST_USE: return "AST_USE";
+		case AST_NAME: return "AST_NAME";
+		case AST_CLOSURE_VAR: return "AST_CLOSURE_VAR";
+		case AST_NULLABLE_TYPE: return "AST_NULLABLE_TYPE";
+		case ZEND_AST_FUNC_DECL: return "AST_FUNC_DECL";
+		case ZEND_AST_CLOSURE: return "AST_CLOSURE";
+		case ZEND_AST_METHOD: return "AST_METHOD";
+		case ZEND_AST_CLASS: return "AST_CLASS";
 		case ZEND_AST_MAGIC_CONST: return "AST_MAGIC_CONST";
 		case ZEND_AST_TYPE: return "AST_TYPE";
 		case ZEND_AST_VAR: return "AST_VAR";
@@ -184,6 +188,7 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_SWITCH: return "AST_SWITCH";
 		case ZEND_AST_SWITCH_CASE: return "AST_SWITCH_CASE";
 		case ZEND_AST_DECLARE: return "AST_DECLARE";
+		case ZEND_AST_PROP_ELEM: return "AST_PROP_ELEM";
 		case ZEND_AST_CONST_ELEM: return "AST_CONST_ELEM";
 		case ZEND_AST_USE_TRAIT: return "AST_USE_TRAIT";
 		case ZEND_AST_TRAIT_PRECEDENCE: return "AST_TRAIT_PRECEDENCE";
@@ -198,11 +203,8 @@ const char *ast_kind_to_name(zend_ast_kind kind) {
 		case ZEND_AST_TRY: return "AST_TRY";
 		case ZEND_AST_CATCH: return "AST_CATCH";
 		case ZEND_AST_PARAM: return "AST_PARAM";
-		case ZEND_AST_PROP_ELEM: return "AST_PROP_ELEM";
 		case ZEND_AST_FOR: return "AST_FOR";
 		case ZEND_AST_FOREACH: return "AST_FOREACH";
-		case AST_NAME: return "AST_NAME";
-		case AST_CLOSURE_VAR: return "AST_CLOSURE_VAR";
 	}
 
 	return NULL;
@@ -220,6 +222,11 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 				case 0: return AST_STR(str_name);
 			}
 			return NULL;
+		case AST_NULLABLE_TYPE:
+			switch (child) {
+				case 0: return AST_STR(str_type);
+			}
+			return NULL;
 		case ZEND_AST_FUNC_DECL:
 			switch (child) {
 				case 0: return AST_STR(str_params);
@@ -678,10 +685,6 @@ zend_string *ast_kind_child_name(zend_ast_kind kind, uint32_t child) {
 }
 
 void ast_register_kind_constants(INIT_FUNC_ARGS) {
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_FUNC_DECL", ZEND_AST_FUNC_DECL, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLOSURE", ZEND_AST_CLOSURE, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_METHOD", ZEND_AST_METHOD, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLASS", ZEND_AST_CLASS, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ARG_LIST", ZEND_AST_ARG_LIST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_LIST", ZEND_AST_LIST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_ARRAY", ZEND_AST_ARRAY, CONST_CS | CONST_PERSISTENT);
@@ -699,6 +702,13 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_NAME_LIST", ZEND_AST_NAME_LIST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TRAIT_ADAPTATIONS", ZEND_AST_TRAIT_ADAPTATIONS, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_USE", ZEND_AST_USE, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_NAME", AST_NAME, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLOSURE_VAR", AST_CLOSURE_VAR, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_NULLABLE_TYPE", AST_NULLABLE_TYPE, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_FUNC_DECL", ZEND_AST_FUNC_DECL, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLOSURE", ZEND_AST_CLOSURE, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_METHOD", ZEND_AST_METHOD, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLASS", ZEND_AST_CLASS, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_MAGIC_CONST", ZEND_AST_MAGIC_CONST, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TYPE", ZEND_AST_TYPE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_VAR", ZEND_AST_VAR, CONST_CS | CONST_PERSISTENT);
@@ -757,6 +767,7 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_SWITCH", ZEND_AST_SWITCH, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_SWITCH_CASE", ZEND_AST_SWITCH_CASE, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_DECLARE", ZEND_AST_DECLARE, CONST_CS | CONST_PERSISTENT);
+	REGISTER_NS_LONG_CONSTANT("ast", "AST_PROP_ELEM", ZEND_AST_PROP_ELEM, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CONST_ELEM", ZEND_AST_CONST_ELEM, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_USE_TRAIT", ZEND_AST_USE_TRAIT, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TRAIT_PRECEDENCE", ZEND_AST_TRAIT_PRECEDENCE, CONST_CS | CONST_PERSISTENT);
@@ -771,9 +782,6 @@ void ast_register_kind_constants(INIT_FUNC_ARGS) {
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_TRY", ZEND_AST_TRY, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_CATCH", ZEND_AST_CATCH, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_PARAM", ZEND_AST_PARAM, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_PROP_ELEM", ZEND_AST_PROP_ELEM, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_FOR", ZEND_AST_FOR, CONST_CS | CONST_PERSISTENT);
 	REGISTER_NS_LONG_CONSTANT("ast", "AST_FOREACH", ZEND_AST_FOREACH, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_NAME", AST_NAME, CONST_CS | CONST_PERSISTENT);
-	REGISTER_NS_LONG_CONSTANT("ast", "AST_CLOSURE_VAR", AST_CLOSURE_VAR, CONST_CS | CONST_PERSISTENT);
 }
