@@ -7,7 +7,7 @@
 extern zend_module_entry ast_module_entry;
 #define phpext_ast_ptr &ast_module_entry
 
-#define PHP_AST_VERSION "0.1.1"
+#define PHP_AST_VERSION "0.1.2"
 
 #ifdef PHP_WIN32
 #	define PHP_AST_API __declspec(dllexport)
@@ -37,8 +37,14 @@ ZEND_EXTERN_MODULE_GLOBALS(ast)
 #define AST_STR(str) AST_G(str)
 
 /* Custom ast kind for names */
-#define AST_NAME 2048
-#define AST_CLOSURE_VAR 2049
+#define AST_NAME          2048
+#define AST_CLOSURE_VAR   2049
+#define AST_NULLABLE_TYPE 2050
+
+/* Pretend it still exists */
+#if PHP_VERSION_ID >= 70100
+# define ZEND_AST_LIST ((1 << (ZEND_AST_IS_LIST_SHIFT + 1)) - 1)
+#endif
 
 extern const size_t ast_kinds_count;
 extern const zend_ast_kind ast_kinds[];
